function [D,psin, beta ] = BioBack2(D, psin, beta, j)
% BioBack2 deletes the requested vector j from a given basis 
%
% Downgrade the corresponding biorthogonal basis and orthonormal basis
%
% Usage:    [Dx, psin, beta ] = BioBack2(D,psin, beta,j);
%
% Inputs:
%   D       given basis
%   psin    given orthonormal basis spanning the same space 
%   beta    biorthogonal basis
%   j       index of the element to be eliminated  
%  
% Outputs:
%   D       reduced basis
%   psin    updated orthonormal basis (downgraded by 1 element)
%   beta    updated biorthogonal basis(downgraded by 1 element)
%
% See also NBioDelete, NBioDictDel, BioDictDel.
%
%       http://www.nonlinear-approx.info/

name='BIOBACK2';    
[N,k]=size(D);
kb=size(beta,2);
if (j<1) | (j>k)
  fprintf('%s: Required index %d is out of range (%d atoms)\n',name,j,k);
  return;
end  

b=beta(:,j);
up=psin(:,j);
beta(:,j)=[];
nor=b'*b;
beta=beta-b*(b'*beta)/nor;
R=psin'*D;
dummy=R(:,j);R(:,j)=[];R(:,kb)=dummy;
for i=j:kb-1
  p=[i i+1];
  [G,R(p,i)]=planerot(R(p,i)); %find appropriate Givens rotation
  R(p,i+1:kb)=G*R(p,i+1:kb);   %apply Givens rotation to R(p,i+1:k)
  psin(:,p)=psin(:,p)*G';      %apply Givens rotation to psin(:,p)
end
%
D(:,j)=[];
psin(:,kb)=[];

%Last modification Laura Rebollo-Neira 2014

%Copyright (C) 2006 Miroslav ANDRLE and Laura REBOLLO-NEIRA
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
