function [cmIa,cDi1,cbeta,c,nCoe]=HBW_BOOMP(cmI,cmIa,Dx,cDi1,c,cbeta,tol,No)
%Hierarquized Block Wise Backward-Optimized Orthogonal Matching Pursuit 
%See Ref 1 for the algorithm
%                                                      
% Inputs:  
%   cmI     cell with the signal (cmI{n} is the signal corresponding to  block n)
%   cmIa    cell with the approximated signal for each block
%   Dx      Dictionary
%   cDi1    cell with indices of the atoms in the approximation of each block.
%   c       cell with the coefficients
%   cbeta   cell of the  biorthogonal atoms.
%   tol     tolerance for the quality of the approximation
%   No      (optional) desired number of atoms in the decomposition if you want really No
%           atoms set tol=0, it speeds the process. 
%  
% Outputs: 
%   cmIa    cell with the updated approximation.
%   cDi1    cell with the updated reduced indices
%   cbeta   cell of updated  biorthogonal atoms
%   c       cell with the updated reduced coefficients. 
%  
% Reference: 
%
%  1-L. Rebollo-Neira, ``Cooperative Orthogonal Matching Pursuit strategies for sparse
%  approximation by partitioning'', arXiv:1501.05971 
%
%  2-M.~Andrle, L.~Rebollo-Neira, E.~Sagianos, ``Backward-optimized orthogonal
%  matching pursuit approach'', IEEE Signal Proc. Let.,11,705--708 (2004).
%
name='HBW_BOOMP';  
tic;
[Lx,Nx]=size(Dx);
NBx=numel(cmI);
NB=NBx;
ND=Nx*NB;
delta=1/(NB*Lx);
dd=[];
dd1=[];
%if nargin==5; tol=1.0e-2;end; 
%if nargin==6; No=0;end;
if No>ND; error('%s:You want to reduce %d atoms into %d atoms!',name,ND,No);end
%initialization
%
nCoe=0;
nb=0;
for i=1:NBx;
nb=nb+1;
iq(nb)=i;
cb=c{nb};
nore(nb)=norm(cmI{i}-cmIa{i})^2;
nCoe = nCoe + numel(cb);
bb=cbeta{nb};
dd=(diag(bb'*bb))';
%========================
[Cp,p]=min((cb.*conj(cb))./dd); %''p'' gives the index of the atoms Di1(p) and Di2(p) which minimize the residual error when eliminated
%Cp is the square error 
   if isempty(Cp)==0 & Cp~=0
    Cpq(nb)=Cp;
    pq(nb)=p;
    else
    Cpq(nb)=10^15;
    pq(nb)=0;
  end
end
totsqe=sum(nore);
%
if (totsqe>tol) & (No==0)
  fprintf('\n%s: Approximation is too coarse, I cannot satisfy the tolerance condition\n',name);
else
 while nCoe > No & totsqe<tol 

     [a,cz]=min(Cpq);
     qb=cz;
     pmin=pq(qb);
     hmin=cmI{iq(qb)};
%
     bb=norm(cbeta{qb}(:,pmin))^2;
     cbeta{qb}=cbeta{qb}-cbeta{qb}(:,pmin)*(cbeta{qb}(:,pmin)'*cbeta{qb})/bb; 
     cbeta{qb}(:,pmin)=[]; %eliminate beta pmin from block qb
     delIa=Dx(:, cDi1{qb}(pmin))*c{qb}(pmin)/(bb);
     c{qb}(pmin)=[];%eliminate coef. pmin from block qb
     cDi1{qb}(pmin)=[]; %eliminate atom  pmin (x) from block qb

     nCoe=nCoe-1;
     c{qb}=cbeta{qb}'*hmin(:);
     h_new=zeros(Lx,1);
     cmIa{iq(qb)}=cmIa{iq(qb)}-delIa;
%
      sqe=norm(cmI{iq(qb)}- cmIa{iq(qb)})^2;
      totsqe=totsqe-nore(qb);
      nore(qb)=sqe;
      totsqe=totsqe+nore(qb);
%------recalculate and store Cp  and pmq
        dd1=[];
       dd1=sum(cbeta{qb}.^2,1);
      Cp=0; p=0;
      if isempty(c{qb})==0
      [Cp,p]=min((c{qb}.*conj(c{qb}))./dd1');
       else
       Cp=10^15;
      end
       Cpq(qb)=Cp;
       pq(qb)=p;
end
end
return
%
%Copyright (C) 2014 Laura REBOLLO-NEIRA
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
