function [cmIa,cDi1,cbeta,c]= HBW_SR_OMP_FFT(cmI,cmIa,Dx,cDi1,cbeta,c,cQ,Mit,eex,icase)
%Hierarquized Swapping Based Refinement of OMP
%See Ref 1 for the algorithms
%
%At each step one atom from the atomic decomposition 
%of one block (donor) is interchanged with another atom from another block (receiver)
%to improve the OMP approximation.
%At each step the biorthogonal vectors (duals of the selected  atoms) are modified.
%The inputs are obtainable from the outputs of the OMP function. 
%
% Inputs:
%  
%   cmI     cell array with the signal- (cmI{n} is a vector with the signal for block n)
%   cmIa    cell array with the approximation of the signal
%   Dx      dictionary
%   cDi1    cell with indices of atoms in the approximation of each block
%   cbeta   cell of biorthogonal vectors (with respect to Dx(:,cDi1) for each block)
%   c       cell with coefficients of the approximation for each block.
%   cQ      cell of the orthonormal basis for the same  subspace
%   Mit     maximum number of swaps. 
%   icase   option for the dictionary:
%           icase =4 for Sin-Cos, icase=2 for Cos icase=3 for Sin dictionaries
%     
% Outputs:
%   cmIa    cell array with the updated approximation after swapping.
%   cDi1    cell with updated indices of atoms in the approximation
%   cbeta   cell of updated biorthogonal vectors  
%   c       cell of  coefficients in the approximation
% 
% References: 
%1- L. Rebollo-Neira,  1-``Cooperative Orthogonal Matching Pursuit strategies for sparse
%  approximation by partitioning'', arXiv:1501.05971
%
%2- M. Andrle and L. Rebollo-Neira, "A swapping-based refinement of orthogonal matching
%   pursuit strategies", Signal Processing, Vol 86, No 3, pp. 480-495, 2006.
%
%  http://www.nonlinear-approx.info/
%
zmax=1;
tol1=1e-07;%default tol1=1e-07;
tol2=1e-10;
tic;
%initialization
[L,N]=size(Dx);
NB=size(cmI,1);
delta=1/(NB*L);
kq=zeros(NB,1);
Cp=zeros(NB,1);
Cpb=zeros(NB,1);
Cpsi=zeros(NB,1);
%
nCoe=0;
for nb=1:NB;
cmI{nb}=cmI{nb}';
cmIa{nb}=cmIa{nb}';
kb(nb)=numel(cDi1{nb});
if kb(nb)~=0
 cD{nb}=Dx(:,cDi1{nb});
else
 cD{nb}=[];
end
k=kb(nb);
iq(nb)=nb;
if isempty(cmIa{nb})==0
Re{nb}=(cmI{nb}-cmIa{nb});
else
Re{nb}=(cmI{nb});
end
nore(nb)=norm(Re{nb})^2;
bb=cbeta{nb};
dd=(diag(bb'*bb))';
cb=c{nb};
nCoe = nCoe + numel(cb);
%========================
if (isempty(cb)==0 | norm(cb)~=0)
[Cpb,p]=min((cb.*conj(cb))./dd); %''p'' gives the index of the atoms Di1(p)  which minimize the residual error when eliminated
    %Cp is the square error 
   Cpq(nb)=Cpb;
   pq(nb)=p;
   pp{nb}=cDi1{nb}(p);
else 
     Cpq(nb)=10^15; %(sloppy, to avoid choosing an empty block)
     pq(nb)=0;
end
%    
clear cc
[max_c,b2,scra]=fast_chose_trig_atom(Re{nb}',N,eex,icase);
 qq{nb}=b2;
   atom=Dx(:,qq{nb});
 if k==0 Q2=atom;
 else
   for p=1:k
      Q2=atom-(cQ{nb}(:,p)'*atom)*cQ{nb}(:,p); %orthogonalization
   end 
%re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1)
   for zi=1:zmax
    for p=1:k
    Q2=Q2-(cQ{nb}(:,p)'*Q2)*cQ{nb}(:,p); %re-orthogonalization
    end
   end
 end
   Cpsi(nb)=abs(Q2'*Re{iq(nb)})/norm(Q2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
z=1;
nore1in=33;%any number >0
nore2in=0;
nore2out=0;
nore1out=0;
fprintf('Initial norm = %g\n',sum(nore));
while (nore1in+nore2in)>(nore1out+nore2out)
    approximation_error_norm=sum(nore);
     [a,bz]=find(Cpq==0);
     Cpq(bz)=10^15; %(sloppy, to avoid choosing an empty block)
     [a,cz]=min(Cpq);
     Cpq(bz)=0;
     qb0=cz;
     pmin=pq(qb0);
     Reini=Re{iq(qb0)};
     nore1in=norm(Reini)^2;
%    
      Q1=cQ{qb0};
      D1=cD{qb0};
      pp{qb0}=cDi1{qb0}(pmin);
      beta1=cbeta{qb0};
      Di11=cDi1{qb0};
      c1=c{qb0};
      h1=cmIa{iq(qb0)};
      qb01=qb0;
      kq1=kb(qb01);
      noreqb01=nore1in;
%
      [cD{qb0},cQ{qb0},cbeta{qb0}]=BioBack2(D1,Q1,beta1,pmin);
%      
%      
      cDi1{qb0}(pmin)=[]; %eliminate atom  pmin (x) from block qb
      kb(qb0)=kb(qb0)-1;
      c{qb0}(pmin)=[];
      c{qb0}=cbeta{qb0}'*cmI{iq(qb0)};
      h_new=zeros(L,1);
      l=numel(c{qb0});
      h_new1=cD{qb0}*c{qb0};
      %----recalculate the approximation with 1 less atom
      for n=1:l
      h_new=h_new+Dx(:,cDi1{qb0}(n))*c{qb0}(n);
      end
      cmIa{iq(qb0)}=h_new;
      Re{iq(qb0)}=(cmI{iq(qb0)}- h_new);
      nore1out=norm(Re{iq(qb0)})^2;
      nore(qb0)=nore1out;
      dd=diag(cbeta{qb0}'*cbeta{qb0});
      if (isempty(c{qb0})==0 | norm(c{qb0})~=0)
%Cpb is the square error 
       [Cpb,p]=min((c{qb0}.*conj(c{qb0}))./dd); %''p'' gives the index of the atoms Di1(p)
%
           Cpq(qb0)=Cpb;
	    pq(qb0)=p;
	    pp{qb0}=cDi1{qb0}(p);
        else
        Cpq(qb0)=10^15; %sloppy  as above 
	pq(qb0)=0;
       end
%     recalculate the Cp for block qb
     [max_c,b2,scra]=fast_chose_trig_atom(Re{iq(qb0)}',N,eex,icase);
      qq{qb0}=b2;
      Cp(qb0)=max_c;
      new_atom2=Dx(:,qq{qb0});
   if kb(qb0)==0 Q2=new_atom2;
   else
        for p=1:kb(qb0)
	   Q2=new_atom2-(cQ{qb0}(:,p)'*new_atom2)*cQ{qb0}(:,p); %orthogonalization
	end
%  %re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1)
        for zi=1:zmax
        for p=1:kb(qb0)
          Q2=Q2-(cQ{qb0}(:,p)'*Q2)*cQ{qb0}(:,p); %re-orthogonalization
        end
        end
   end
        Cpsi(qb0)=abs(Q2'*Re{iq(qb0)})/norm(Q2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     [Cpmax,qb]=max(Cpsi);
      q(1)=qq{qb};
      if Cpmax<tol2
      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
        break;
      end
  kb(qb)=kb(qb)+1;
  k=kb(qb);
  New_Atom=Dx(:,q(1));
%  k2=k-1;
  Q2=cQ{qb};
  beta2=cbeta{qb};
  h2=cmIa{iq(qb)};
  Di12=cDi1{qb};
  f=cmI{iq(qb)};
  c2=c{qb};
  D2=cD{qb};
  nore2in=norm(Re{iq(qb)})^2;
  qb2=qb;
  kq2=kb(qb);
    [cQ{qb},cbeta{qb}]=BioFor2(New_Atom,Q2,beta2,k);
    cD{qb}(:,k)=New_Atom;
    cDi1{qb}(k)=q(1);
    f=cmI{iq(qb)};
    c{qb}=cbeta{qb}'*cmI{iq(qb)};
    h_new2=zeros(L,1);
    l=numel(c{qb});
   %----recalculate the approximation with 1 less atom
    for n=1:l
      h_new2=h_new2+ Dx(:,cDi1{qb}(n))*c{qb}(n);
    end
      cmIa{iq(qb)}=h_new2;
      Re{iq(qb)}=cmI{iq(qb)}- h_new2;
      nore2out= norm(Re{iq(qb)})^2;
      nore(qb)=nore2out;
%     
%     recalculate the Cp for block qb
     [max_c,b2,scra]=fast_chose_trig_atom(Re{iq(qb)}',N,eex,icase);
      qq{qb}=b2;
      Cp(qb)=max_c;
     new_atom2=Dx(:,qq{qb}(1));
     for p=1:kb(qb)
      Q2=new_atom2-(cQ{qb}(:,p)'*new_atom2)*cQ{qb}(:,p); %orthogonalization
     end
%re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1)
     for zi=1:zmax
     for p=1:kb(qb)
        Q2=Q2-(cQ{qb}(:,p)'*Q2)*cQ{qb}(:,p); %re-orthogonalization
     end
     end
     Cpsi(qb)=abs(Q2'*Re{iq(qb)})/norm(Q2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       [Cpmax,qb]=max(Cpsi);
       q(1)=qq{qb}(1);
       if Cpmax<tol2
        fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
       break;
       end
  if (qb0==qb & pp{qb}==q) fprintf('Stop\n');z=z;break; end %check
  if z > Mit; break; end
  nor1=nore1in+nore2in;
  nor2=nore1out+nore2out;
  difnor12=nor1-nor2;
%   fprintf('z=%d, %f, %f, %f\n',z, (nor1), (nor2),sum(nore));
  if (nor1)<=(nor2) 
    fprintf('\n nor1<=nor2\n');
    fprintf('z=%d, %f, %f, %f\n\n',z, (nor1), (nor2),sum(nore));
    cD{qb01}=D1;
    cQ{qb01}=Q1;
    cbeta{qb01}=beta1;
    cDi1{qb01}=Di11;
    cmIa{iq(qb01)}=h1;
    c{qb01}=c1;
    nore(qb01)=norm(cmIa{iq(qb01)}-cmI{iq(qb01)})^2;
    if qb01~=qb2
    cD{qb2}=D2;
    cQ{qb2}=Q2;
    cbeta{qb2}=beta2;
    cDi1{qb2}=Di12;
    cmIa{iq(qb2)}=h2;
    c{qb2}=c2;
    nore(qb2)=norm(cmIa{iq(qb2)}-cmI{iq(qb2)})^2;
    end
    noredif2=nore2in;
    z=z-1;
    fprintf('number of swaps z=%d,  error norm %f\n',z, sum(nore));
%     
  end
z=z+1;
end
%
%Copyright (C) Laura Rebollo-Neira (2014). 
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
